@@ -231,6 +231,7 @@ class LogicParser(object):
         @raise LogicParseError: if there is a syntax error in the logic
         """
         def flatten(l): 
+            """Flatten out a list which contains both values and sublists"""
             ret = []
             for item in l:
                 if isinstance(item, list):
@@ -462,10 +463,11 @@ class BinaryConstraint(CodedConstraint):
      - <=       (less than or equal to)
      - >=       (greater than or equal to)
      - LIKE     (same as equal to, but with implied wildcards)
+     - CONTAINS (same as equal to, but with implied wildcards)
      - NOT LIKE (same as not equal to, but with implied wildcards)
 
     """
-    OPS = set(['=', '!=', '<', '>', '<=', '>=', 'LIKE', 'NOT LIKE'])
+    OPS = set(['=', '!=', '<', '>', '<=', '>=', 'LIKE', 'NOT LIKE', 'CONTAINS'])
     def __init__(self, path, op, value, code="A"):
         """
         Constructor
@@ -487,9 +489,17 @@ class BinaryConstraint(CodedConstraint):
         super(BinaryConstraint, self).__init__(path, op, code)
 
     def to_string(self):
+        """
+        Provide a human readable representation of the logic. 
+        This method is called by repr.
+        """
         s = super(BinaryConstraint, self).to_string()
         return " ".join([s, str(self.value)])
     def to_dict(self):
+        """
+        Return a dict object which can be used to construct a 
+        DOM element with the appropriate attributes.
+        """
         d = super(BinaryConstraint, self).to_dict()
         d.update(value=str(self.value))
         return d
@@ -518,9 +528,17 @@ class ListConstraint(CodedConstraint):
         super(ListConstraint, self).__init__(path, op, code)
 
     def to_string(self):
+        """
+        Provide a human readable representation of the logic. 
+        This method is called by repr.
+        """
         s = super(ListConstraint, self).to_string()
         return " ".join([s, str(self.list_name)])
     def to_dict(self):
+        """
+        Return a dict object which can be used to construct a 
+        DOM element with the appropriate attributes.
+        """
         d = super(ListConstraint, self).to_dict()
         d.update(value=str(self.list_name))
         return d
@@ -537,6 +555,9 @@ class LoopConstraint(CodedConstraint):
      - IS
      - IS NOT
 
+    The operators IS and IS NOT map to the ops "=" and "!=" when they
+    are used in XML serialisation.
+
     """
     OPS = set(['IS', 'IS NOT'])
     SERIALISED_OPS = {'IS':'=', 'IS NOT':'!='}
@@ -561,67 +582,216 @@ class LoopConstraint(CodedConstraint):
         super(LoopConstraint, self).__init__(path, op, code)
 
     def to_string(self):
+        """
+        Provide a human readable representation of the logic. 
+        This method is called by repr.
+        """
         s = super(LoopConstraint, self).to_string()
         return " ".join([s, self.loopPath])
     def to_dict(self):
+        """
+        Return a dict object which can be used to construct a 
+        DOM element with the appropriate attributes.
+        """
         d = super(LoopConstraint, self).to_dict()
         d.update(loopPath=self.loopPath, op=self.SERIALISED_OPS[self.op])
         return d
     
 class TernaryConstraint(BinaryConstraint):
+    """
+    Constraints for broad, general searching over all fields
+    ========================================================
+
+    These constraints request a wide-ranging search for matching
+    fields over all aspects of an object, including up to coercion
+    from related classes.
+
+    Valid operators:
+     - LOOKUP
+
+    To aid disambiguation, Ternary constaints accept an extra_value as 
+    well as the main value.
+    """
     OPS = set(['LOOKUP'])
     def __init__(self, path, op, value, extra_value=None, code="A"):
+        """
+        Constructor
+        ===========
+
+        @param path: The path to constrain. Here is must be a class, or a reference to a class.
+        @type path: string
+
+        @param op: The relationship between the path and the path provided (must be a valid operator)
+        @type op: string
+
+        @param value: The value to check other fields against.
+        @type value: string
+
+        @param extra_value: A further value for disambiguation. The meaning of this value varies by class
+                            and configuration. For example, if the class of the object is Gene, then
+                            extra_value will refer to the Organism. 
+        @type value: string
+
+        @param code: The code for this constraint (default = "A")
+        @type code: string
+        """
         self.extra_value = extra_value
         super(TernaryConstraint, self).__init__(path, op, value, code)
 
     def to_string(self):
+        """
+        Provide a human readable representation of the logic. 
+        This method is called by repr.
+        """
         s = super(TernaryConstraint, self).to_string()
         if self.extra_value is None:
             return s
         else:
             return " ".join([s, 'IN', self.extra_value])
     def to_dict(self):
+        """
+        Return a dict object which can be used to construct a 
+        DOM element with the appropriate attributes.
+        """
         d = super(TernaryConstraint, self).to_dict()
         if self.extra_value is not None:
             d.update(extraValue=self.extra_value)
         return d
 
 class MultiConstraint(CodedConstraint):
+    """
+    Constraints for checking membership of a set of values
+    ======================================================
+
+    These constraints require the value they constrain to be
+    either a member of a set of values, or not a member.
+
+    Valid operators:
+     - ONE OF
+     - NONE OF
+
+    These constraints are similar in use to List constraints, with
+    the following differences:
+      - The list in this case is a defined set of values that is passed
+        along with the query itself, rather than anything stored
+        independently on a server.
+      - The object of the constaint is the value of an attribute, rather
+        than an object's identity.
+    """
     OPS = set(['ONE OF', 'NONE OF'])
     def __init__(self, path, op, values, code="A"):
-        if not isinstance(values, list):
-            raise TypeError("values must be a list, not " + str(type(values)))
+        """
+        Constructor
+        ===========
+
+        @param path: The path to constrain. Here it must be an attribute of some object.
+        @type path: string
+
+        @param op: The relationship between the path and the path provided (must be a valid operator)
+        @type op: string
+
+        @param values: The set of values which the object of the constraint either must or must not belong to.
+        @type value: string
+
+        @param code: The code for this constraint (default = "A")
+        @type code: string
+        """
+        if not isinstance(values, (set, list)):
+            raise TypeError("values must be a set or a list, not " + str(type(values)))
         self.values = values
         super(MultiConstraint, self).__init__(path, op, code)
 
     def to_string(self):
+        """
+        Provide a human readable representation of the logic. 
+        This method is called by repr.
+        """
         s = super(MultiConstraint, self).to_string()
         return ' '.join([s, str(self.values)])
     def to_dict(self):
+        """
+        Return a dict object which can be used to construct a 
+        DOM element with the appropriate attributes.
+        """
         d = super(MultiConstraint, self).to_dict()
         d.update(value=self.values)
         return d
 
 class SubClassConstraint(Constraint):
+    """
+    Constraints on the class of a reference
+    =======================================
+
+    If an object has a reference X to another object of type A, 
+    and type B extends type A, then any object of type B may be 
+    the value of the reference X. If you only want to see X's
+    which are B's, this may be achieved with subclass constraints, 
+    which allow the type of an object to be limited to one of the 
+    subclasses (at any depth) of the class type required 
+    by the attribute.
+
+    These constraints do not use operators. Since they cannot be 
+    conditional (eg. "A is a B or A is a C" would not be possible
+    in an InterMine query), they do not have codes 
+    and cannot be referenced in logic expressions.
+    """
     def __init__(self, path, subclass):
-       if not PATH_PATTERN.match(subclass):
-            raise TypeError
-       self.subclass = subclass
-       super(SubClassConstraint, self).__init__(path)
+        """
+        Constructor
+        ===========
+
+        @param path: The path to constrain. This must refer to a class or a reference to a class.
+        @type path: str
+
+        @param subclass: The class to subclass the path to. This must be a simple class name (not a dotted name)
+        @type subclass: str
+        """
+        if not PATH_PATTERN.match(subclass):
+             raise TypeError
+        self.subclass = subclass
+        super(SubClassConstraint, self).__init__(path)
     def to_string(self):
+       """
+       Provide a human readable representation of the logic. 
+       This method is called by repr.
+       """
        s = super(SubClassConstraint, self).to_string()
        return s + ' ISA ' + self.subclass
     def to_dict(self):
+       """
+       Return a dict object which can be used to construct a 
+       DOM element with the appropriate attributes.
+       """
        d = super(SubClassConstraint, self).to_dict()
        d.update(type=self.subclass) 
        return d
 
 
 class TemplateConstraint(object):
+    """
+    A mixin to supply the behaviour and state of constraints on templates
+    =====================================================================
+
+    Constraints on templates can also be designated as "on", "off" or "locked", which refers
+    to whether they are active or not. Inactive constraints are still configured, but behave
+    as if absent for the purpose of results. In addition, template constraints can be
+    editable or not. Only values for editable constraints can be provided when requesting results,
+    and only constraints that can participate in logic expressions can be editable.
+    """
     REQUIRED = "locked"
     OPTIONAL_ON = "on"
     OPTIONAL_OFF = "off"
     def __init__(self, editable=True, optional="locked"):
+        """
+        Constructor
+        ===========
+
+        @param editable: Whether or not this constraint should accept new values.
+        @type editable: bool
+        
+        @param optional: Whether a value for this constraint must be provided when running.
+        @type optional: "locked", "on" or "off"
+        """
         self.editable = editable
         if optional == TemplateConstraint.REQUIRED:
             self.optional = False
@@ -637,13 +807,26 @@ class TemplateConstraint(object):
 
     @property
     def required(self):
+        """
+        True if a value must be provided for this constraint.
+
+        @rtype: bool
+        """
         return not self.optional
 
     @property
     def switched_off(self):
+        """
+        True if this constraint is currently inactive.
+
+        @rtype: bool
+        """
         return not self.switched_on
 
     def get_switchable_status(self):
+        """
+        Returns either "locked", "on" or "off".
+        """
         if not self.optional:
             return "locked"
         else:
@@ -651,9 +834,23 @@ class TemplateConstraint(object):
             return switch
 
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         editable = "editable" if self.editable else "non-editable"
         return '(' + editable + ", " + self.get_switchable_status() + ')'
     def separate_arg_sets(self, args):
+        """
+        A static function to use when building template constraints. 
+        ------------------------------------------------------------
+
+        dict -> (dict, dict)
+
+        Splits a dictionary of arguments into two separate dictionaries, one with
+        arguments for the main constraint, and one with arguments for the template
+        portion of the behaviour
+        """
         c_args = {}
         t_args = {}
         for k, v in args.items():
@@ -671,6 +868,10 @@ class TemplateUnaryConstraint(UnaryConstraint, TemplateConstraint):
         UnaryConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(UnaryConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
@@ -680,6 +881,10 @@ class TemplateBinaryConstraint(BinaryConstraint, TemplateConstraint):
         BinaryConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(BinaryConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
@@ -689,6 +894,10 @@ class TemplateListConstraint(ListConstraint, TemplateConstraint):
         ListConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(ListConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
@@ -698,6 +907,10 @@ class TemplateLoopConstraint(LoopConstraint, TemplateConstraint):
         LoopConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(LoopConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
@@ -707,6 +920,10 @@ class TemplateTernaryConstraint(TernaryConstraint, TemplateConstraint):
         TernaryConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(TernaryConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
@@ -716,6 +933,10 @@ class TemplateMultiConstraint(MultiConstraint, TemplateConstraint):
         MultiConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(MultiConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
@@ -725,23 +946,54 @@ class TemplateSubClassConstraint(SubClassConstraint, TemplateConstraint):
         SubClassConstraint.__init__(self, *a, **c_args)
         TemplateConstraint.__init__(self, **t_args)
     def to_string(self):
+        """
+        Provide a template specific human readable representation of the 
+        constraint. This method is called by repr.
+        """
         return(SubClassConstraint.to_string(self) 
                 + " " + TemplateConstraint.to_string(self))
 
 class ConstraintFactory(object):
+    """
+    A factory for creating constraints from a set of arguments.
+    ===========================================================
 
+    A constraint factory is responsible for finding an appropriate
+    constraint class for the given arguments and instantiating the 
+    constraint.
+    """
     CONSTRAINT_CLASSES = set([
         UnaryConstraint, BinaryConstraint, TernaryConstraint, 
         MultiConstraint, SubClassConstraint, LoopConstraint,
         ListConstraint])
 
     def __init__(self):
+        """
+        Constructor
+        -----------
+
+        Creates a new ConstraintFactory
+        """
         self._codes = iter(string.ascii_uppercase)
     
     def get_next_code(self):
+        """
+        Return the available constraint code.
+
+        @return: A single uppercase character
+        @rtype: str
+        """
         return self._codes.next()
 
     def make_constraint(self, *args, **kwargs):
+        """
+        Create a constraint from a set of arguments.
+        --------------------------------------------
+
+        Finds a suitable constraint class, and instantiates it.
+
+        @rtype: Constraint
+        """
         for CC in self.CONSTRAINT_CLASSES:
             try:
                 c = CC(*args, **kwargs)
@@ -753,6 +1005,15 @@ class ConstraintFactory(object):
             + str(args) + ", " + str(kwargs))
     
 class TemplateConstraintFactory(ConstraintFactory):
+    """
+    A factory for creating constraints with template specific characteristics.
+    ==========================================================================
+
+    A constraint factory is responsible for finding an appropriate
+    constraint class for the given arguments and instantiating the 
+    constraint. TemplateConstraintFactories make constraints with the 
+    extra set of TemplateConstraint qualities.
+    """
     CONSTRAINT_CLASSES = set([
         TemplateUnaryConstraint, TemplateBinaryConstraint, 
         TemplateTernaryConstraint, TemplateMultiConstraint,
